/*
 * $Header: /home/cvs/jakarta-slide/src/webdav/server/org/apache/slide/webdav/WebdavServletConfig.java,v 1.3 2001/08/20 12:13:14 cmlenz Exp $
 * $Revision: 1.3 $
 * $Date: 2001/08/20 12:13:14 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.slide.webdav;

import java.util.Enumeration;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;

/**
 * WebDAV Servlet Configuration. This class wraps around a ServletConfig object
 * and makes the parameters conveniently accessible to the WebdavServlet and 
 * the WebdavMethod subclasses.
 * 
 * @see org.apache.slide.webdav.method.WebdavMethod#getConfig
 * 
 * @author Christopher Lenz (cmlenz at apache.org)
 */
public class WebdavServletConfig
    implements ServletConfig {
    
    
    // -------------------------------------------------------------- Constants
    
    
    private static final String DEFAULT_SERVLET_PARAMETER =
        "default-servlet";
    
    
    private static final String DEPTH_LIMIT_PARAMETER =
        "depth-limit";
    
    
    private static final String DEFAULT_MIME_TYPE_PARAMETER =
        "default-mime-type";
    
    
    private static final String SCOPE_PARAMETER =
        "scope";
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * The wrapped ServletConfig.
     */
    private ServletConfig config;
    
    
    /**
     * Whether or not the servlet is mapped as default servlet of the web 
     * application. This has quite drastic consequences for the mapping of
     * request URIs (the path-info is then returned by getServletPath() instead
     * of getPathInfo()), but there is no safe way for the servlet to determine
     * whether it's mapped as default servlet. For this reason we need an extra
     * initialization parameter to tell us whether we're the default servlet
     * or not.
     *
     * Because the Slide WebdavServlet is usually deployed as default servlet, 
     * this setting defaults to <code>true</code>
     */
    private boolean isDefaultServlet = true;
    
    
    /**
     * Default MIME type of resources.
     */
    private String defaultMimeType = "text/plain";
    
    
    /**
     * Depth limit. To limit tree browsing when using depth = infinity.
     */
    private int depthLimit = 3;
    
    
    /**
     * Default MIME type of resources.
     */
    private String scope = "";
    
    
    // ----------------------------------------------------------- Construction
    
    
    /**
     * Constructor.
     *
     * @param config ServletConfig
     */
    WebdavServletConfig(ServletConfig config) {
        
        this.config = config;
        
        ServletContext context = getServletContext();
        String value = null;
        
        // read 'scope' parameter
        try {
            value = getInitParameter(SCOPE_PARAMETER);
            if (value == null) {
                value = context.getInitParameter(SCOPE_PARAMETER);
            }
            if (value != null) {
                scope = value;
            }
        } catch (Throwable t) {
            ;
        }
        
        // read 'depth-limit' parameter
        try {
            value = getInitParameter(DEPTH_LIMIT_PARAMETER);
            if (value == null) {
                value = context.getInitParameter(DEPTH_LIMIT_PARAMETER);
            }
            if (value != null) {
                depthLimit = Integer.parseInt(value);
            }
        } catch (Throwable t) {
            ;
        }
        
        // read 'default-mime-type' parameter
        try {
            value = getInitParameter(DEFAULT_MIME_TYPE_PARAMETER);
            if (value == null) {
                value = context.getInitParameter(DEFAULT_MIME_TYPE_PARAMETER);
            }
            if (value != null) {
                defaultMimeType = value;
            }
        } catch (Throwable t) {
            ;
        }
        
        // read 'default-servlet' parameter
        try {
            value = getInitParameter(DEFAULT_SERVLET_PARAMETER);
            if (value != null) {
                isDefaultServlet = Boolean.valueOf(value).booleanValue();
            }
        } catch (Throwable t) {
            ;
        }
        
    }
    
    
    // ------------------------------------------- ServletConfig Implementation
    
    
    /**
     * Returns a String containing the value of the named initialization 
     * parameter, or null if the parameter does not exist.
     *
     * @param name a String specifying the name of the initialization parameter
     * @return a String containing the value of the initialization parameter
     */
    public String getInitParameter(String name) {
        
        return config.getInitParameter(name);
    }
    
    
    /**
     * Returns the names of the servlet's initialization parameters as an 
     * Enumeration of String objects, or an empty Enumeration if the servlet 
     * has no initialization parameters.
     *
     * @return an Enumeration of String objects containing the names of the 
     *         servlet's initialization parameters
     */
    public Enumeration getInitParameterNames() {
        
        return config.getInitParameterNames();
    }
    
    
    /**
     * Returns a reference to the ServletContext in which the caller 
     * is executing.
     *
     * @return a ServletContext object, used by the caller to interact with 
     *         its servlet container
     */
    public ServletContext getServletContext() {
        
        return config.getServletContext();
    }
    
    
    /**
     * Returns the name of this servlet instance.
     * The name may be provided via server administration, assigned in the 
     * web application deployment descriptor, or for an unregistered (and thus 
     * unnamed) servlet instance it will be the servlet's class name.
     *
     * @return the name of the servlet instance
     */
    public String getServletName() {
        
        return config.getServletName();
    }
    
    
    // --------------------------------------------------------- Public Methods
    
    
    /**
     * Returns the default MIME type of resources, to be used for example when
     * clients do not specify the content-type of files uploaded with the 
     * PUT method.
     *
     * @return the default MIME type (for example &quot;text/plain&quot;)
     */
    public String getDefaultMimeType() {
        
        return defaultMimeType;
    }
    
    
    /**
     * Returns the depth limit. The depth limit should be used to limit the 
     * depth of (for instance) PROPFIND operations, in case the client has 
     * requested &quot;Depth: infinity&quot;
     *
     * @return the depth limit
     */
    public int getDepthLimit() {
        
        return depthLimit;
    }
    
    
    /**
     * Returns the user defined scope in the namespace that shall be exposed
     * by the WebdavServlet. This should be treated as root directory of the
     * WebDAV namespace. By default this will be an empty string.
     *
     * @return the scope URI
     */
    public String getScope() {
        
        return scope;
    }
    
    
    /**
     * Returns whether the servlet is mapped as default servlet of the web 
     * application.
     * 
     * @returns true if the servlet is configured as default servlet of the
     *          context
     */
    public boolean isDefaultServlet() {
        
        return isDefaultServlet;
    }
    
    
}

